@@ -29,7 +29,8 @@ from dkist_header_validator.exceptions import ReturnTypeException
 from dkist_header_validator.exceptions import SpecSchemaDefinitionException
 from dkist_header_validator.exceptions import SpecValidationException
 from dkist_header_validator.exceptions import ValidationException
-from dkist_header_validator.translator import convert_spec122_to_spec214
+from dkist_header_validator.translator import translate_spec122_to_spec214
+from dkist_header_validator.translator import translate_spec122_to_spec214_l0
 
 
 logger = logging.getLogger(__name__)
@@ -382,7 +383,7 @@ class SpecValidator:
         verified_headers = self.verify_headers(headers, extra)
         return verified_headers, fits_cards
 
-    def _validate_file(self, input_headers: Union[str, IO], extra) -> Tuple[dict, dict]:
+    def _validate_file(self, input_headers: Union[str, IO], extra) -> Tuple[dict, dict, np.ndarray]:
         """
         Validates files against the astropy and then instance spec_schema
         :param input_headers: The input headers to validate in the following formats:
@@ -402,11 +403,13 @@ class SpecValidator:
                 # normalize headers into a dict
                 try:
                     hdus = self._headers_to_dict(hdul[1].header)
+                    data = hdul[1].data
                 except IndexError:  # non-compressed
                     hdus = self._headers_to_dict(hdul[0].header)
+                    data = hdul[0].data
                 fits_cards = self._capture_fits_cards(hdus)
                 verified_headers = self.verify_headers(hdus, extra)
-                return verified_headers, fits_cards
+                return verified_headers, fits_cards, data
         except (ValueError, FileNotFoundError, OSError, IndexError) as exc:
             logger.error(f"Cannot parse headers: detail = {exc}")
             raise ValidationException(f"Cannot parse headers", errors={type(exc): str(exc)})
@@ -423,7 +426,7 @@ class SpecValidator:
         if isinstance(validated_headers, HDUList):
             return validated_headers
         temp_array: np.ndarray = np.ones((1, 1, 1), dtype=np.int16)
-        new_hdu = fits.PrimaryHDU(temp_array)
+        new_hdu = fits.PrimaryHDU(temp_array, header=fits.Header())
         for (key, value) in validated_headers.items():
             new_hdu.header[key] = value
         for key in fits_cards:
@@ -445,76 +448,93 @@ class SpecValidator:
         return validated_headers
 
     @staticmethod
-    def _return_BytesIO(validated_headers, input_headers, fits_cards) -> BytesIO:
+    def _return_BytesIO(validated_headers, input_headers, data, fits_cards) -> BytesIO:
         """
         Returns validated headers as a BytesIO object
         :param validated_headers: Already validated/translated headers to be written out into
           the BytesIO object
-        :param input_headers: original filepath
+        :param input_headers: original filepath or BytesIO object
+        :param data: original data
         :param fits_cards: Any special cards to be included in the BytesIO object
         :return: BytesIO object
         """
-        if isinstance(input_headers, pathlib.PosixPath):
-            hdul = fits.open(input_headers)
-            if hdul[0].data == None:  # check if data is in second HDU
-                new_hdu = fits.PrimaryHDU(hdul[1].data)
-            else:
-                new_hdu = fits.PrimaryHDU(hdul[0].data)
-            for (key, value) in validated_headers.items():
-                new_hdu.header[key] = value
-            for key in fits_cards:
-                new_hdu.header[key] = str(fits_cards[key])
-            new_hdu_list = fits.HDUList([new_hdu])
-            return BytesIO(
-                new_hdu_list.writeto(
-                    str(os.path.basename(input_headers)),
-                    overwrite=True,
-                    output_verify="exception",
-                    checksum=True,
-                )
-            )
-        raise ReturnTypeException("No data. Cannot write BytesIO object.")
-
-    @staticmethod
-    def _return_file(validated_headers, input_headers, fits_cards) -> (str, IO):
-        """
-        Returns validated headers as a FITS file
-        :param validated_headers: Already validated/translated headers to be written out
-          into a FITS file
-        :param input_headers: original filepath
-        :param fits_cards: Any special cards to be included in the FITS file
-        :return: FITS file
-        """
-        if isinstance(input_headers, pathlib.PosixPath):
-            hdul = fits.open(input_headers)
-            if hdul[0].data == None:  # check if data is in second HDU
-                new_hdu = fits.PrimaryHDU(hdul[1].data)
-            else:
-                new_hdu = fits.PrimaryHDU(hdul[0].data)
-            for (key, value) in validated_headers.items():
-                new_hdu.header[key] = value
-            for key in fits_cards:
-                new_hdu.header[key] = str(fits_cards[key])
-            new_hdu_list = fits.HDUList([new_hdu])
+        new_hdu = fits.PrimaryHDU(data, header=fits.Header())
+        for (key, value) in validated_headers.items():
+            new_hdu.header[key] = value
+        for key in fits_cards:
+            new_hdu.header[key] = str(fits_cards[key])
+        new_hdu_list = fits.HDUList([new_hdu])
+        return BytesIO(
             new_hdu_list.writeto(
                 str(os.path.basename(input_headers)),
                 overwrite=True,
                 output_verify="exception",
                 checksum=True,
             )
-            return os.path.basename(input_headers)
-        raise ReturnTypeException("No data. Cannot write file.")
+        )
 
-    def _format_output(self, return_type, validated_headers, input_headers=None, fits_cards=None):
+    @staticmethod
+    def _return_file(validated_headers, input_headers, data, fits_cards) -> (str, IO):
+        """
+        Returns validated headers as a FITS file
+        :param validated_headers: Already validated/translated headers to be written out
+          into a FITS file
+        :param input_headers: original filepath or BytesIO object
+        :param data: original data
+        :param fits_cards: Any special cards to be included in the FITS file
+        :return: FITS file
+        """
+        new_hdu = fits.PrimaryHDU(data, header=fits.Header())
+        for (key, value) in validated_headers.items():
+            new_hdu.header[key] = value
+        for key in fits_cards:
+            new_hdu.header[key] = str(fits_cards[key])
+        new_hdu_list = fits.HDUList([new_hdu])
+        new_hdu_list.writeto(
+            str(os.path.basename(input_headers)),
+            overwrite=True,
+            output_verify="exception",
+            checksum=True,
+        )
+        return os.path.basename(input_headers)
+
+    @staticmethod
+    def _return_HDU(validated_headers, data, fits_cards):
+        """
+        Returns validated headers as an HDU
+        :param validated_headers: Already validated/translated headers to be written out
+          into a FITS file
+        :param data: original data
+        :param fits_cards: Any special cards to be included in the FITS file
+        :return: HDU
+        """
+        new_hdu = fits.PrimaryHDU(data, header=fits.Header())
+        for (key, value) in validated_headers.items():
+            new_hdu.header[key] = value
+        for key in fits_cards:
+            new_hdu.header[key] = str(fits_cards[key])
+        return new_hdu
+
+    def _format_output(
+        self, return_type, validated_headers, input_headers=None, data=None, fits_cards=None
+    ):
         fits_cards = fits_cards or {}
         if return_type == Path:
-            return self._return_file(validated_headers, input_headers, fits_cards)
+            if data is None:
+                raise ReturnTypeException("No data. Cannot write file.")
+            return self._return_file(validated_headers, input_headers, data, fits_cards)
         if return_type == BytesIO:
-            return self._return_BytesIO(validated_headers, input_headers, fits_cards)
+            if data is None:
+                raise ReturnTypeException("No data. Cannot write BytesIO object.")
+            return self._return_BytesIO(validated_headers, input_headers, data, fits_cards)
         if return_type == dict:
             return self._return_dictionary(validated_headers, fits_cards)
-        if return_type == HDUList or fits.header.Header:
+        if return_type == HDUList or return_type == fits.header.Header:
             return self._return_hdulist(validated_headers, fits_cards)
+        if return_type == fits.PrimaryHDU:
+            if data is None:
+                raise ReturnTypeException("No data. Cannot write PrimaryHDU.")
+            return self._return_HDU(validated_headers, data, fits_cards)
 
     def _capture_fits_cards(self, validated_headers) -> dict:
         """
@@ -548,6 +568,7 @@ class SpecValidator:
             - fits.header.Header
             - Path (file)
             - HDUList
+            - fits.PrimaryHDU
         :param extra: switch for validation to allow extra keys in schema. Default is true, which will
         allow extra keys. Ingest validation should allow extra keys.
         :return: formatted headers
@@ -555,13 +576,15 @@ class SpecValidator:
         """
         if isinstance(input_headers, (dict, fits.header.Header, HDUList)):
             validated_headers, fits_cards = self._validate_headers(input_headers, extra)
-            return self._format_output(return_type, validated_headers, input_headers, fits_cards)
-        validated_headers, fits_cards = self._validate_file(input_headers, extra)
-        return self._format_output(return_type, validated_headers, input_headers, fits_cards)
+            return self._format_output(
+                return_type, validated_headers, input_headers, None, fits_cards
+            )
+        validated_headers, fits_cards, data = self._validate_file(input_headers, extra)
+        return self._format_output(return_type, validated_headers, input_headers, data, fits_cards)
 
-    def validate_and_translate(self, input_headers, return_type=HDUList, extra=True):
+    def validate_and_translate_to_214_l0(self, input_headers, return_type=HDUList, extra=True):
         """
-        Validates against the instance spec_schema and then translates
+        Validates against the instance spec_schema and then translates to the spec214_l0 schema
         :param input_headers: The headers to validate in the following formats:
             - string file path
             - File like object
@@ -574,16 +597,51 @@ class SpecValidator:
             - fits.header.Header
             - Path (file)
             - HDUList
+            - fits.PrimaryHDU
         :param extra: switch for validation to allow extra keys in schema. Default is true, which will
         allow extra keys. Ingest validation should allow extra keys.
-        :return: formatted headers
+        :return: formatted 214 l0 headers
         :raises: SpecValidationException or subclass
         """
         if isinstance(input_headers, (dict, fits.header.Header, HDUList)):
             validated_headers, fits_cards = self._validate_headers(input_headers, extra)
-            translated_headers = convert_spec122_to_spec214(validated_headers)
-            return self._format_output(return_type, translated_headers, fits_cards)
+            translated_headers = translate_spec122_to_spec214_l0(validated_headers)
+            return self._format_output(return_type, translated_headers, None, None, fits_cards)
         else:
-            validated_headers, fits_cards = self._validate_file(input_headers, extra)
-            translated_headers = convert_spec122_to_spec214(validated_headers)
-            return self._format_output(return_type, translated_headers, input_headers, fits_cards)
+            validated_headers, fits_cards, data = self._validate_file(input_headers, extra)
+            translated_headers = translate_spec122_to_spec214_l0(validated_headers)
+            return self._format_output(
+                return_type, translated_headers, input_headers, data, fits_cards
+            )
+
+    def validate_and_translate_to_214(self, input_headers, return_type=HDUList, extra=True):
+        """
+        Validates against the instance spec_schema and then translates to the full spec214 schema
+        :param input_headers: The headers to validate in the following formats:
+            - string file path
+            - File like object
+            - HDUList object
+            - fits.header.Header object
+            - Dictionary of header keys and values
+        :param return_type: determines return type. Default is HDUList. May be one of:
+            - dict
+            - BytesIO
+            - fits.header.Header
+            - Path (file)
+            - HDUList
+            - fits.PrimaryHDU
+        :param extra: switch for validation to allow extra keys in schema. Default is true, which will
+        allow extra keys. Ingest validation should allow extra keys.
+        :return: formatted 214 headers
+        :raises: SpecValidationException or subclass
+        """
+        if isinstance(input_headers, (dict, fits.header.Header, HDUList)):
+            validated_headers, fits_cards = self._validate_headers(input_headers, extra)
+            translated_headers = translate_spec122_to_spec214(validated_headers)
+            return self._format_output(return_type, translated_headers, None, None, fits_cards)
+        else:
+            validated_headers, fits_cards, data = self._validate_file(input_headers, extra)
+            translated_headers = translate_spec122_to_spec214(validated_headers)
+            return self._format_output(
+                return_type, translated_headers, input_headers, data, fits_cards
+            )
