@@ -42,6 +42,17 @@ def test_spec214_return_fits_header(valid_spec_214_header):
     spec214_validator.validate(valid_spec_214_header, return_type=fits.header.Header, extra=False)
 
 
+def test_spec214_return_HDU(valid_spec_214_file):
+    """
+    Validates a fits header against the SPEC-0214 schema
+    Given: A valid SPEC-0214 fits header
+    When: Validating headers
+    Then: return validated fits.PrimaryHDU object and do not raise an exception
+    """
+    # raises exception on failure
+    spec214_validator.validate(valid_spec_214_file, return_type=fits.PrimaryHDU, extra=False)
+
+
 def test_spec214_return_BytesIO(valid_spec_214_file):
     """
     Validates a fits header against the SPEC-0214 schema
@@ -776,6 +787,7 @@ def valid_l0_spec_214_headers(tmpdir_factory):
         "OBSPR_ID": "JX3O8NXFI6FGTVZ1D7G7U8OVUWDZQL",
         "DKIST004": "dark",
         "WAVELNTH": 582.3,
+        "LINEWAV": 582.3,
         "EXPER_ID": "1XXPIDR5CEXMZ0SQ8LT3HMF83FW4HJ",
         "ID___012": "1XXPIDR5CEXMZ0SQ8LT3HMF83FW4HJ",
         "OBJECT": "quietsun",
@@ -840,6 +852,7 @@ def valid_l0_spec_214_headers(tmpdir_factory):
         "CAM__005": 126.3,
         "CAM__004": 13.2,
         "CAM__014": 3,
+        "FILE_ID": "RANDOMSTRING",
     }
 
     temp_dir = tmpdir_factory.mktemp("valid l0_214_headers_temp")
