@@ -19,16 +19,41 @@ from dkist_fits_specifications.utils import expand_naxis
 
 logger = logging.getLogger(__name__)
 
-__all__ = ["convert_spec122_to_spec214"]
+__all__ = ["translate_spec122_to_spec214_l0", "translate_spec122_to_spec214"]
 
 type_map = {"int": int, "float": float, "str": str, "bool": bool}
 
 
-def convert_spec122_to_spec214(
+def translate_spec122_to_spec214_l0(
     spec122_input: Union[HDUList, dict, fits.header.Header, str, IO, List]
 ) -> Union[dict, HDUList]:
     """
-    Convert spec 122 headers to spec 214 headers
+    Convert spec 122 headers to spec 214 l0 headers
+    :param spec122_input: Spec 122 headers or headers + data to convert
+    """
+    # extract headers and data
+    input_headers, input_data = _parse_spec122(spec122_input)
+    # convert headers
+    output_headers = _add_214_l0_headers(input_headers)
+    # update DATE keyword
+    output_headers["DATE"] = datetime.now().isoformat()
+    # remove BZERO and BSCALE keywords- we don't use these anymore
+    try:
+        del output_headers["BZERO"]
+        del output_headers["BSCALE"]
+    except KeyError:
+        # if they aren't there we can't delete them
+        pass
+    if input_data:  # return hdu list if the input had data
+        return _format_output_hdu(output_headers, input_data)
+    return output_headers  # return headers if only headers were given
+
+
+def translate_spec122_to_spec214(
+    spec122_input: Union[HDUList, dict, fits.header.Header, str, IO, List]
+) -> Union[dict, HDUList]:
+    """
+    Convert spec 122 headers to spec 214 l0 headers
     :param spec122_input: Spec 122 headers or headers + data to convert
     """
     # extract headers and data
@@ -82,15 +107,36 @@ def _format_output_hdu(hdr, data) -> HDUList:
     return hdu_list
 
 
-generic_spec214_schema = level0.load_level0_spec214().values()
+# generic_spec214_schema = level0.load_level0_spec214().values()
 
 
-def _create_spec214_schema_for_header(hdr: dict) -> Dict[str, Dict[str, Any]]:
-    s = {
-        key: schema for definition in generic_spec214_schema for (key, schema) in definition.items()
-    }
-    hdr["DNAXIS"] = hdr["NAXIS"]
-    return expand_index_d(expand_naxis(hdr["NAXIS"], s), DNAXIS=hdr["DNAXIS"])
+def _create_spec214_schema_for_header(
+    hdr: dict, generic_schema, naxis=None
+) -> Dict[str, Dict[str, Any]]:
+    s = {key: schema for definition in generic_schema for (key, schema) in definition.items()}
+    if naxis == None:
+        return s
+    else:
+        hdr["DNAXIS"] = naxis
+        return expand_index_d(expand_naxis(hdr["NAXIS"], s), DNAXIS=hdr["DNAXIS"])
+
+
+def _add_214_l0_headers(hdr: dict):
+    """
+    Translates 122 keywords to 214 l0 keywords and returns a dictionary
+    """
+    result = {}  # output headers
+    generic_spec214_l0_schema = level0.load_level0_spec214().values()
+    spec214_l0_schema = _create_spec214_schema_for_header(hdr, generic_spec214_l0_schema)
+
+    # translate 122 -> 214 headers
+    for key, key_schema in spec214_l0_schema.items():
+        result.update(_translate_key(key, key_schema, hdr))
+
+    # add remaining header values to result
+    hdr_keys_not_translated = {k: v for k, v in hdr.items() if k not in result}
+    result.update(hdr_keys_not_translated)
+    return result
 
 
 def _add_214_headers(hdr: dict):
@@ -98,7 +144,8 @@ def _add_214_headers(hdr: dict):
     Adds new 214 headers into dictionary
     """
     result = {}  # output headers
-    spec214_schema = _create_spec214_schema_for_header(hdr)
+    generic_spec214_schema = load_full_spec214().values()
+    spec214_schema = _create_spec214_schema_for_header(hdr, generic_spec214_schema, hdr["NAXIS"])
 
     # translate 122 -> 214 headers
     for key, key_schema in spec214_schema.items():
@@ -111,7 +158,7 @@ def _add_214_headers(hdr: dict):
 
 
 def _translate_key(key, key_schema, hdr) -> dict:
-    default_values = {"str": "default", "int": -999, "float": -999.9}
+    default_values = {"str": "default", "int": -999, "float": -999.9, "bool": False}
     key_is_copied = key_schema.get("copy")
     copy_schema_only = key_schema.get("copy") == "schema"
     key_is_renamed = key_schema.get("rename")
